from configparser import ConfigParser
import os
import numpy as np
import pandas as pd
import pathlib
import pdb
import ast

class PhotoSex_config(object):
    '''read or write configuration file for PhotoSex'''
    
    def __init__(self,path_config_file='./',name_config_file='PhotoSex_config.ini'):
        self.cfile=path_config_file+'/'+name_config_file
        config_object = ConfigParser()
        self.config_object=config_object
        self.homedir=str(pathlib.Path.home())+'/'
        self.workdir_default=path_config_file
    
    def create_config_file(self):
        self.config_object['BASIC']={
            'show_computer_info':True
        }
        
        self.config_object['PATHS']={
            'workdir':self.workdir_default,
            'sexdir':self.homedir+'Science/data/models/sextractor/',
        }
        
        self.config_object['PHOTOMETRY']={
            'apertures':[0.5,1.,1.5,2.,3.,4.,5.,6.,7.,8.,12.,18]
        }
        
        self.config_object['APERTURE_CORRECTION']={
            'apc_polyorder':0,
            'apc_radius_total_flux':100,
            'apc_checkplot_curveofgrowth':False
        }
        
        self.config_object['CHECK_PHOT_PROCESS']={
            'checkpro_check_fits_image':False,
            'checkpro_check_png_plot':False,
            'checkpro_checkpng_width':12.
        }
        
        self.config_object['CHECK_PHOT_RESULT']={
            'checkres_reference_flag_threshold':128,
            'checkres_zmagfit_outliers_method':0,
            'checkres_zmagfit_outliers_nsig_dbscan':1.5,
            'checkres_zmagfit_outliers_minfrac_dbscan':0.5,
            'checkres_zmagfit_outliers_nsig_sigmaclip':3.,
            'checkres_zmagfit_polyorder':0,
            'checkres_plot_rawimage_scalerange':[0.1,99.],
            'checkres_plot_zoominimage_width':60.,
            'checkres_plot_zoominimage_scalerange':[0.1,97],
            'checkres_keep_html_only':True
        }
        
        with open(self.cfile, 'w') as conf:
            self.config_object.write(conf)
        #pdb.set_trace()    
        return 1
    
    def read_config(self):
        self.config_object.read(self.cfile)
        photosex_info={'phot_show_env':self.config_object.getboolean('BASIC','show_computer_info'),
                       'workdir':self.config_object['PATHS']['workdir'],
                       'sexdir':self.config_object['PATHS']['sexdir'],
                       'order_apc':self.config_object.getint('APERTURE_CORRECTION','apc_polyorder'),
                       'apc_rad_tot':self.config_object.getfloat('APERTURE_CORRECTION','apc_radius_total_flux'),
                       'checkcog_apc':self.config_object.getboolean('APERTURE_CORRECTION',
                                                                    'apc_checkplot_curveofgrowth'),
                       'checkimg':self.config_object.getboolean('CHECK_PHOT_PROCESS','checkpro_check_fits_image'),
                       'checkplot':self.config_object.getboolean('CHECK_PHOT_PROCESS','checkpro_check_png_plot'),
                       'checkfig_width':self.config_object.getfloat('CHECK_PHOT_PROCESS','checkpro_checkpng_width'),
                       'checkphot_zmag_order':self.config_object.getint('CHECK_PHOT_RESULT','checkres_zmagfit_polyorder'),
                       'checkphot_flag_ref_threshold':self.config_object.getfloat('CHECK_PHOT_RESULT',
                                                                                  'checkres_reference_flag_threshold'),
                       'checkphot_if_cali_outliers_detection_method':self.config_object.getint('CHECK_PHOT_RESULT',
                                                                                               'checkres_zmagfit_outliers_method'),
                       'checkphot_imgraw_interval':ast.literal_eval(self.config_object.get('CHECK_PHOT_RESULT',
                                                                                           'checkres_plot_rawimage_scalerange')),
                       'checkphot_imgzoom_interval':ast.literal_eval(self.config_object.get('CHECK_PHOT_RESULT',
                                                                                            'checkres_plot_zoominimage_scalerange')),
                       'checkphot_nsig_dbscan':self.config_object.getfloat('CHECK_PHOT_RESULT',
                                                                           'checkres_zmagfit_outliers_nsig_dbscan'),
                       'checkphot_minfrac_dbscan':self.config_object.getfloat('CHECK_PHOT_RESULT',
                                                                              'checkres_zmagfit_outliers_minfrac_dbscan'),
                       'checkphot_nsig_sigmaclip':self.config_object.getfloat('CHECK_PHOT_RESULT',
                                                                              'checkres_zmagfit_outliers_nsig_sigmaclip'),
                       'checkphot_keep_html_only':self.config_object.getboolean('CHECK_PHOT_RESULT',
                                                                                'checkres_keep_html_only'),
                       'apers':ast.literal_eval(self.config_object.get('PHOTOMETRY','apertures')),
                       'checkphot_zoom_width':self.config_object.getfloat('CHECK_PHOT_RESULT',
                                                                          'checkres_plot_zoominimage_width')}
        
        return photosex_info
    
    def setconfig(self):
        if os.path.isfile(self.cfile) == False:
            self.create_config_file()
            photosex_info=self.read_config()
        else:
            photosex_info=self.read_config()
        return photosex_info



from astropy.io import ascii
from astropy.table import Table

import sys
import re
import numpy as np
import pylab
import json
import requests

try: # Python 3.x
    from urllib.parse import quote as urlencode
    from urllib.request import urlretrieve
except ImportError:  # Python 2.x
    from urllib import pathname2url as urlencode
    from urllib import urlretrieve

try: # Python 3.x
    import http.client as httplib 
except ImportError:  # Python 2.x
    import httplib  
    
def ps1cone(ra,dec,radius,table="mean",release="dr1",format="csv",columns=None,
           baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False,
           **kw):
    """Do a cone search of the PS1 catalog
    
    Parameters
    ----------
    ra (float): (degrees) J2000 Right Ascension
    dec (float): (degrees) J2000 Declination
    radius (float): (degrees) Search radius (<= 0.5 degrees)
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    format: csv, votable, json
    columns: list of column names to include (None means use defaults)
    baseurl: base URL for the request
    verbose: print info about request
    **kw: other parameters (e.g., 'nDetections.min':2)
    """
    
    data = kw.copy()
    data['ra'] = ra
    data['dec'] = dec
    data['radius'] = radius
    return ps1search(table=table,release=release,format=format,columns=columns,
                    baseurl=baseurl, verbose=verbose, **data)


def ps1search(table="mean",release="dr1",format="csv",columns=None,
           baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False,
           **kw):
    """Do a general search of the PS1 catalog (possibly without ra/dec/radius)
    
    Parameters
    ----------
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    format: csv, votable, json
    columns: list of column names to include (None means use defaults)
    baseurl: base URL for the request
    verbose: print info about request
    **kw: other parameters (e.g., 'nDetections.min':2).  Note this is required!
    """
    
    data = kw.copy()
    if not data:
        raise ValueError("You must specify some parameters for search")
    checklegal(table,release)
    if format not in ("csv","votable","json"):
        raise ValueError("Bad value for format")
    url = "{baseurl}/{release}/{table}.{format}".format(**locals())
    if columns:
        # check that column values are legal
        # create a dictionary to speed this up
        dcols = {}
        for col in ps1metadata(table,release)['name']:
            dcols[col.lower()] = 1
        badcols = []
        for col in columns:
            if col.lower().strip() not in dcols:
                badcols.append(col)
        if badcols:
            raise ValueError('Some columns not found in table: {}'.format(', '.join(badcols)))
        # two different ways to specify a list of column values in the API
        # data['columns'] = columns
        data['columns'] = '[{}]'.format(','.join(columns))

# either get or post works
#    r = requests.post(url, data=data)
    r = requests.get(url, params=data)

    if verbose:
        print(r.url)
    r.raise_for_status()
    if format == "json":
        return r.json()
    else:
        return r.text


def checklegal(table,release):
    """Checks if this combination of table and release is acceptable
    
    Raises a VelueError exception if there is problem
    """
    
    releaselist = ("dr1", "dr2")
    if release not in ("dr1","dr2"):
        raise ValueError("Bad value for release (must be one of {})".format(', '.join(releaselist)))
    if release=="dr1":
        tablelist = ("mean", "stack")
    else:
        tablelist = ("mean", "stack", "detection")
    if table not in tablelist:
        raise ValueError("Bad value for table (for {} must be one of {})".format(release, ", ".join(tablelist)))


def ps1metadata(table="mean",release="dr1",
           baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs"):
    """Return metadata for the specified catalog and table
    
    Parameters
    ----------
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    baseurl: base URL for the request
    
    Returns an astropy table with columns name, type, description
    """
    
    checklegal(table,release)
    url = "{baseurl}/{release}/{table}/metadata".format(**locals())
    r = requests.get(url)
    r.raise_for_status()
    v = r.json()
    # convert to astropy table
    tab = Table(rows=[(x['name'],x['type'],x['description']) for x in v],
               names=('name','type','description'))
    return tab


def mastQuery(request):
    """Perform a MAST query.

    Parameters
    ----------
    request (dictionary): The MAST request json object

    Returns head,content where head is the response HTTP headers, and content is the returned data
    """
    
    server='mast.stsci.edu'

    # Grab Python Version 
    version = ".".join(map(str, sys.version_info[:3]))

    # Create Http Header Variables
    headers = {"Content-type": "application/x-www-form-urlencoded",
               "Accept": "text/plain",
               "User-agent":"python-requests/"+version}

    # Encoding the request as a json string
    requestString = json.dumps(request)
    requestString = urlencode(requestString)
    
    # opening the https connection
    conn = httplib.HTTPSConnection(server)

    # Making the query
    conn.request("POST", "/api/v0/invoke", "request="+requestString, headers)

    # Getting the response
    resp = conn.getresponse()
    head = resp.getheaders()
    content = resp.read().decode('utf-8')

    # Close the https connection
    conn.close()

    return head,content


def resolve(name):
    """Get the RA and Dec for an object using the MAST name resolver
    
    Parameters
    ----------
    name (str): Name of object

    Returns RA, Dec tuple with position"""

    resolverRequest = {'service':'Mast.Name.Lookup',
                       'params':{'input':name,
                                 'format':'json'
                                },
                      }
    headers,resolvedObjectString = mastQuery(resolverRequest)
    resolvedObject = json.loads(resolvedObjectString)
    # The resolver returns a variety of information about the resolved object, 
    # however for our purposes all we need are the RA and Dec
    try:
        objRa = resolvedObject['resolvedCoordinate'][0]['ra']
        objDec = resolvedObject['resolvedCoordinate'][0]['decl']
    except IndexError as e:
        raise ValueError("Unknown object '{}'".format(name))
    return (objRa, objDec)



import numpy as np
import os
import time
import pandas as pd
from astropy.io import fits
from astropy.table import Table
import math
import pdb
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
from astropy import units as u
from astropy.wcs.utils import proj_plane_pixel_scales as pixel_scale
from matplotlib.patches import Rectangle
from tqdm import tqdm
from subprocess import Popen, PIPE, STDOUT
import gc
from sklearn.cluster import DBSCAN

class basic(object):
    def axis2map(nx,ny,subval=None):
        if subval==None:
            subval=0.
        x=np.array(np.arange(nx)-subval).reshape(1,-1)
        y=np.array(np.arange(ny)-subval).reshape(-1,1)
        #print(x)
        #print(y)
        mx=np.repeat(x,ny,axis=0)
        my=np.repeat(y,nx,axis=1)
        return mx,my
    def gcirc(ra1,dc1,ra2,dc2):
        #input coordinate must be in decimal degrees
        d2r=np.pi/180.
        as2r=np.pi/648000.
        h2r=np.pi/12.0
        
        rarad1=ra1*d2r
        rarad2=ra2*d2r
        dcrad1=dc1*d2r
        dcrad2=dc2*d2r
        
        deldec2=(dcrad2-dcrad1)/2.0
        delra2=(rarad2-rarad1)/2.0
        sindis=(np.sin(deldec2)*np.sin(deldec2)+\
                np.cos(dcrad1)*np.cos(dcrad2)*np.sin(delra2)*np.sin(delra2))**0.5
        dis=2.0*np.arcsin(sindis)
        
        dis=dis/as2r
        return dis#in arcsecs
    
    def spawn(cmd,silent=False):
        import subprocess
        try:
            output=subprocess.check_output(cmd,shell=True,stderr=subprocess.STDOUT)
            if silent == False:
                if (not isinstance(output, int)):
                    print(output.decode('utf-8'))
        except subprocess.CalledProcessError as e:
            output=e.output
            code=e.returncode
            if silent == False:
                if (not isinstance(code,int)) and (not isinstance(output,int)):
                    print(code.decode('utf-8'),output.decode('utf-8'))
        return output
    
    def extinction_law_mydef(select='wang19'):
        if select == 'xue16':
            ks={'j':2.72, 'h':1.6, 'k':1.0, 
                'irac1':0.553, 'irac2':0.461, 'irac3':0.389, 'irac4': 0.426, 'mips24':0.264,
                'w1':0.591, 'w2':0.463, 'w3':0.537, 'w4':0.364,
                'v':1./0.112}
        
        if select == 'rieke85':
            ks = {'u':1.531, 'b':1.324, 'v':1.0, 'r':0.748, 'i':0.482, 
                  'j':0.282, 'h':0.175, 'k':0.112, 'l':0.058, 'm':0.023, 'n':0.052,
                  '8.0':0.02, '8.5':0.043, '9.0':0.074, '9.5':0.087,
                  '10.0':0.083, '10.5':0.074, '11.0':0.060, '11.5':0.047, '12.0':0.037, '12.5':0.030, '13.0':0.027}
    
        if select == 'ind05':
            ks={'j':2.50, 'h':1.55, 'k':1.0, 
                'irac1':0.56, 'irac2':0.43, 'irac3':0.43, 'irac4': 0.43,
                'w1':0.56, 'w2':0.43, 'w3':0.43,
                'v':1./0.112}
        
        if select == 'wang19':
            ks0={'g_bp':1.002, 'g_rp':0.589, 'b':1.317, 
                'v':1.0, 'u_sdss':1.584, 'g_sdss':1.205,
                'r_sdss':0.848, 'i_sdss':0.630, 'z_sdss':0.458,
                'g_pan':1.155, 'r_pan':0.843, 'i_pan':0.628,
                'z_pan':0.487, 'y_pan':0.395, 'j':0.243,
                'h':0.131, 'k':0.078, 'w1':0.039,
                'w2':0.026, 'w3':0.04, 'g_gaia':0.789,
                'irac1':0.037, 'irac2':0.026, 'irac3':0.019,
                'irac4':0.025}
            ks=ks0.copy()
            factor=ks0['k']
            for kkk in ks0:
                ks[kkk]=ks0[kkk]/factor
            
            
        return ks
    
    def range_extend(range_in,extend=0.1):
        xmin=range_in[0]
        xmax=range_in[1]
        size_in=xmax-xmin
        range_out=np.array([xmin-size_in*extend,xmax+size_in*extend])
        return range_out
    
    def convert_dfcol_int2float(df1):
        ncol=df1.shape[1]
        for icol in np.arange(ncol):
            cond=(df1.dtypes[icol].type is np.int) | (df1.dtypes[icol].type is np.int16) | \
                 (df1.dtypes[icol].type is np.int32) | (df1.dtypes[icol].type is np.int64)
            #print(cond)
            #pdb.set_trace()
            if cond is True:
                #print(df1.columns[icol],df1.dtypes[icol])
                df1[df1.columns[icol]]=df1[df1.columns[icol]].astype(float)
        return df1
    
    def convert_tabcol_int2float(tab):
        cols=tab.colnames
        for icol in cols:
            cond=(tab[icol].dtype.type is np.int) | (tab[icol].dtype.type is np.int16) | \
                 (tab[icol].dtype.type is np.int32) | (tab[icol].dtype.type is np.int64)
            if cond is True:
                tab[icol]=tab[icol].astype(float)
        return tab
    
    def crossmatch_2dfs(df1,df2,coo_cols=['ra','dec','RAJ2000','DEJ2000'],
                        frame='icrs',maxdist=None):
        c1=SkyCoord(df1[coo_cols[0]].values*u.degree,df1[coo_cols[1]].values*u.degree,frame=frame)
        c2=SkyCoord(df2[coo_cols[2]].values*u.degree,df2[coo_cols[3]].values*u.degree,frame=frame)
        idx, dist2d, dist3d = c1.match_to_catalog_sky(c2)
        df2s=df2.loc[idx,:]
        df2s.reset_index(inplace=True)
        df1.reset_index(inplace=True)
        df1.drop(columns=['index'],inplace=True)
        df2s.drop(columns=['index'],inplace=True)
        dfc=pd.concat([df1,df2s],axis=1)
        #pdb.set_trace()
        dfc.insert(dfc.columns.size,'d_arcsec',dist2d.value*3600.)
        if maxdist is not None:
            dfc=dfc[dfc['d_arcsec']<maxdist]
        return dfc
    
    
    def str2float_one(string):
        try:
            f=np.float(string)
        except:
            f=np.nan
        return f
    
    def syscmd(cmd, encoding=''):
        """
        Runs a command on the system, waits for the command to finish, and then
        returns the text output of the command. If the command produces no text
        output, the command's return code will be returned instead.
        """
        p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT,
            close_fds=True)
        p.wait()
        output = p.stdout.read()
        if len(output) > 1:
            if encoding: return output.decode(encoding)
            else: return output
        return p.returncode
    
    def patch_slit(ra_cen,dec_cen,wcs=None,slit_len=7*u.arcmin,slit_width=1*u.arcsec,slit_posang=45.,
                   edgecolor='red',facecolor='none'):
        ra=ra_cen
        dec=dec_cen
        if wcs is None:
            raise ValueError('Please offer wcs!')
        subwcs=wcs
        pixscl=pixel_scale(subwcs)[0]*3600.
        xc,yc=subwcs.wcs_world2pix(ra,dec,0)
        width=slit_len.to(u.arcsec).value/pixscl
        height=slit_width.to(u.arcsec).value/pixscl
        posang=slit_posang
        posang_rad=posang/180*np.pi
        xim=xc - ((width / 2) * np.cos(posang_rad)) + ((height / 2) * np.sin(posang_rad))
        yim=yc - ((width / 2) * np.sin(posang_rad)) - ((height / 2) * np.cos(posang_rad))
        rr=Rectangle((xim,yim), width, height,
                     angle=posang,
                     edgecolor=edgecolor, facecolor=facecolor,zorder=10)
        #pdb.set_trace()
        return rr
    
    def outliers_detection_dbscan(df, cols=['pmra','pmdec'],nsig=2.5,min_cluster_size_fraction=0.5):
        dfuse=df.copy()
        pmra=(dfuse[cols[0]].values-dfuse[cols[0]].median())/dfuse[cols[0]].std()
        pmdec=(dfuse[cols[1]].values-dfuse[cols[1]].median())/dfuse[cols[1]].std()
        X=np.vstack([pmra,pmdec]).T
        num_diff=1
        labels=np.zeros(X.shape[0])
        nsig_dbscan=nsig
        
        while num_diff > 0:
            no_noise_before=labels[labels==-1].shape[0]
            x0=X[:,0]
            x1=X[:,1]
            std0=np.nanstd(x0[labels>-1])
            std1=np.nanstd(x1[labels>-1])
            #pdb.set_trace()
            stdmax=np.nanmax([std0,std1])
            epsilon=stdmax*nsig_dbscan#2.85
            min_samples=x0[labels>-1].shape[0]*min_cluster_size_fraction
            db = DBSCAN(eps=epsilon, min_samples=min_samples).fit(X)
            labels = db.labels_
            no_clusters = len(np.unique(labels) )
            no_noise = np.sum(np.array(labels) == -1, axis=0)
            #print(no_noise)
            num_diff=no_noise-no_noise_before
    #pdb.set_trace()
        dfuse.insert(dfuse.columns.size,'clusters',labels)
        dfuse.insert(dfuse.columns.size,'Outlier','N')
        dfuse.loc[dfuse['clusters']==-1,'Outlier']='Y'
        return dfuse
    
    def df_to_little_endian(df):
        dftem=df.copy()
        for col in dftem.columns:
            bo=dftem[col].values.dtype.byteorder
            if bo == '>':
                dftem[col]=dftem[col].values.byteswap().newbyteorder()
        return dftem




from __future__ import print_function
import numpy
from astropy.table import Table
import requests
from PIL import Image
from io import BytesIO
import pylab
import pdb

class download_panstar_image(object):
    def __init__(self, ra=83.633210, dec=22.014460, size=1280, filters='grizy', format='jpg',imgtype='warp'):
        self.ra=ra
        self.dec=dec
        self.size=size
        self.filters=filters
        self.format=format
        self.imgtype=imgtype
        
    def getimages(self):
    
        """Query ps1filenames.py service to get a list of images
    
        ra, dec = position in degrees
        size = image size in pixels (0.25 arcsec/pixel)
        filters = string with filters to include
        Returns a table with the results
        """
    
        service = "https://ps1images.stsci.edu/cgi-bin/ps1filenames.py"
        url = service+("?ra={ra}&dec={dec}&size={size}&format=fits"
               "&filters={filters}&type={imgtype}").format(ra=self.ra,dec=self.dec,
                                                        size=self.size,filters=self.filters,imgtype=self.imgtype)
        table = Table.read(url, format='ascii')
        return table


    def geturl(self, output_size=None, color=False):
    
        """Get URL for images in the table
    
        ra, dec = position in degrees
        size = extracted image size in pixels (0.25 arcsec/pixel)
        output_size = output (display) image size in pixels (default = size).
                      output_size has no effect for fits format images.
        filters = string with filters to include
        format = data format (options are "jpg", "png" or "fits")
        color = if True, creates a color image (only for jpg or png format).
                Default is return a list of URLs for single-filter grayscale images.
        Returns a string with the URL
        """
    
        if color and self.format == "fits":
            raise ValueError("color images are available only for jpg or png formats")
        if self.format not in ("jpg","png","fits"):
            raise ValueError("format must be one of jpg, png, fits")
        table = self.getimages()
        url = ("https://ps1images.stsci.edu/cgi-bin/fitscut.cgi?"
               "ra={ra}&dec={dec}&size={size}&format={format}&type={imgtype}").format(ra=self.ra,dec=self.dec,
                                                                                      size=self.size,
                                                                                      format=self.format,
                                                                                      imgtype=self.imgtype)
        if output_size:
            url = url + "&output_size={}".format(output_size)
        # sort filters from red to blue
        flist = ["yzirg".find(x) for x in table['filter']]
        table = table[numpy.argsort(flist)]
        if color:
            if len(table) > 3:
                # pick 3 filters
                table = table[[0,len(table)//2,len(table)-1]]
            for i, param in enumerate(["red","green","blue"]):
                url = url + "&{}={}".format(param,table['filename'][i])
        else:
            urlbase = url + "&red="
            url = []
            for filename in table['filename']:
                url.append(urlbase+filename)
        #pdb.set_trace()
        return url


    def getcolorim(self, output_size=None):
    
        """Get color image at a sky position
    
        ra, dec = position in degrees
        size = extracted image size in pixels (0.25 arcsec/pixel)
        output_size = output (display) image size in pixels (default = size).
                  output_size has no effect for fits format images.
        filters = string with filters to include
        format = data format (options are "jpg", "png")
        Returns the image
        """
    
        if self.format not in ("jpg","png"):
            raise ValueError("format must be jpg or png")
        url = self.geturl(output_size=output_size, color=True)
        r = requests.get(url)
        im = Image.open(BytesIO(r.content))
        return im


    def getgrayim(self, output_size=None):
    
        """Get grayscale image at a sky position
    
        ra, dec = position in degrees
        size = extracted image size in pixels (0.25 arcsec/pixel)
        output_size = output (display) image size in pixels (default = size).
                      output_size has no effect for fits format images.
        filter = string with filter to extract (one of grizy)
        format = data format (options are "jpg", "png")
        Returns the image
        """
    
        if self.format not in ("jpg","png"):
            raise ValueError("format must be jpg or png")
        if self.filters not in list("grizy"):
            raise ValueError("filter must be one of grizy")
        url = self.geturl(output_size=output_size)
        r = requests.get(url[0])
        im = Image.open(BytesIO(r.content))
        return im



import sys
import os
#from distutils.core import setup
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="SimplePhotoSex",
    version="0.1",
    packages=["download_panstar", "PhotoSex"],
    package_dir={"download_panstar": "download_panstar","PhotoSex":"PhotoSex"},
    #package_data={"pnicer": ["tests_resources/*.fits"]},
    #install_requires=["numpy>=1.11", "scipy>=0.18", "scikit-learn>=0.18", "matplotlib>=1.5", "astropy>=1.3"],
    url="https://gitee.com/zhang-miaomiao1983/photsex",
    license="MIT",
    author="Miaomiao Zhang",
    author_email="mmzhang83@163.com",
    description="A simple tool to perform aperture or PSF photometry with Sextractor+PSFEX",
    long_description=read('README.md'),
    long_description_content_type='text/plain',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from __future__ import print_function
import os
import numpy as np
from astropy.table import Table
from astropy.io import ascii
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
import astropy.units as u
import requests
from PIL import Image
from io import BytesIO
import pylab
from download_panstar.download_panstar_image import download_panstar_image as dpi
from download_panstar import download_panstar_catalog as dpc
import pdb
import matplotlib.pyplot as plt
from matplotlib.colors import LogNorm
from matplotlib.colors import PowerNorm
import matplotlib.colors as mcolors
import matplotlib.cm as cm
from astropy.io import fits
from astropy.visualization import PercentileInterval, AsinhStretch
from astropy.wcs import WCS

class download_panstar(object):
    '''Download PanSTARRS1 image and catalog based on central position'''
    def __init__(self, coo_cen_skycoord, size_pixel, 
                 name='test', imgtype='stack', band3color='grz', band='r', outdir=None):
        
        coo_icrs=coo_cen_skycoord.transform_to('icrs')
        ra=coo_icrs.ra.value
        dec=coo_icrs.dec.value
        size = size_pixel
        radius=size*0.25/3600.
        self.coo_icrs=coo_icrs
        self.ra=ra
        self.dec=dec
        self.size=size
        self.radius=radius
        self.name=name
        self.imgtype=imgtype
        self.band3color=band3color
        self.band=band
        self.outdir=outdir
        self.cat_types=['mean','stack','detection']

    def image(self,fitsname=None):

        fits_filter=self.band
        if fits_filter == 'g':
            filterid=1
        elif fits_filter == 'r':
            filterid=2
        elif fits_filter == 'i':
            filterid=3
        elif fits_filter == 'z':
            filterid=4
        elif fits_filter == 'y':
            filterid=5
        else:
            raise ValueError('There is no filter of '+fits_filter+' in PanSTARRS1 survey.')
    
        if fitsname is None:
            fitsname=self.name+'_'+fits_filter+'_panstar.fits'
        #catname=fitsname.replace('_'+fits_filter+'_','_cat_')
        fitsurl = dpi(ra=self.ra,dec=self.dec,size=self.size,filters=fits_filter, 
                      format="fits",imgtype=self.imgtype).geturl()
        fh = fits.open(fitsurl[0])
        fh.writeto(self.outdir+fitsname,overwrite=True)
        data,hdr=fits.getdata(self.outdir+fitsname,0,header=True)
        if 'PC001001' in hdr:
            #hdr['CD1_1']=hdr['PC001001']
            #hdr['CD1_2']=hdr['PC001002']
            #hdr['CD2_1']=hdr['PC002001']
            #hdr['CD2_2']=hdr['PC002002']
            #del hdr['PC001001']
            #del hdr['PC001002']
            #del hdr['PC002001']
            #del hdr['PC002002']
            pass
        if 'GAIN' not in hdr:
            hdr['GAIN']=hdr['HIERARCH CELL.GAIN']
        if 'SATURATE' not in hdr:
            hdr['SATURATE'] = hdr['HIERARCH CELL.SATURATION']
        #pdb.set_trace()
        fits.writeto(self.outdir+fitsname,data,hdr,overwrite=True)
        hdu=fits.PrimaryHDU(data=data,header=hdr)
        return hdu,fitsname
    
    def catalog(self,cattype='mean', wcs=None):
        if cattype == 'stack':
            sconstraints = {'primaryDetection':1}
            scolumns = """objID,raMean,decMean,nDetections,ng,nr,ni,nz,ny,
                nStackDetections,primaryDetection,qualityFlag,
                gPSFMag,rPSFMag,iPSFMag,zPSFMag,yPSFMag,
                gPSFMagErr,rPSFMagErr,iPSFMagErr,zPSFMagErr,yPSFMagErr,
                ginfoFlag,rinfoFlag,iinfoFlag,zinfoFlag,yinfoFlag,
                ginfoFlag2,rinfoFlag2,iinfoFlag2,zinfoFlag2,yinfoFlag2,
                ginfoFlag3,rinfoFlag3,iinfoFlag3,zinfoFlag3,yinfoFlag3""".split(',')
        elif cattype == 'detection':
            sconstraints = {'filterID':filterid}
            scolumns = """objID,imageID,filterID,surveyID,obsTime,expTime,airMass,ra,dec,zp,psfFlux,psfFluxErr,
                 psfMajorFWHM,psfMinorFWHM,
                 psfTheta,psfCore,psfQf,psfQfPerfect,psfChiSq,psfLikelihood,apFlux,
                 apFluxErr,apFillF,apRadius,kronFlux,kronFluxErr,kronRad,sky,skyErr,
                 infoFlag,infoFlag2,infoFlag3""".split(',')
        elif cattype == 'mean':
            sconstraints = {'nDetections.gt':1}
            scolumns = """objID,raMean,decMean,nDetections,ng,nr,ni,nz,ny,
                qualityFlag,
                gMeanPSFMag,rMeanPSFMag,iMeanPSFMag,zMeanPSFMag,yMeanPSFMag,
                gMeanPSFMagErr,rMeanPSFMagErr,iMeanPSFMagErr,zMeanPSFMagErr,yMeanPSFMagErr,
                gFlags,rFlags,iFlags,zFlags,yFlags""".split(',')
        else:
            raise ValueError('Only support catalog types of '+self.cat_types)
        
        # strip blanks and weed out blank and commented-out values
        scolumns = [x.strip() for x in scolumns]
        scolumns = [x for x in scolumns if x and not x.startswith('#')]

        sresults = dpc.ps1cone(self.ra,self.dec,self.radius,table=cattype,
                               release="dr2",columns=scolumns,verbose=True,**sconstraints)
        #pdb.set_trace()
        stab = ascii.read(sresults)
        
        for col in scolumns:
            try:
                stab[col]
            except KeyError:
                print(col,"not found")
        # improve the format
        if cattype == 'mean':
            for filter in 'grizy':
                col = filter+'MeanPSFMag'
                try:
                    stab[col].format = ".4f"
                    stab[col][stab[col] == -999.0] = np.nan
                except KeyError:
                    print("{} not found".format(col))
        elif cattype == 'stack':
            for filter in 'grizy':
                col = filter+'PSFMag'
                try:
                    stab[col].format = ".4f"
                    stab[col][stab[col] == -999.0] = np.nan
                except KeyError:
                    print("{} not found".format(col))
        else:
            pass

        if 'objID' in stab.columns:
            stab['objID']=stab['objID'].astype(str)
        df=stab.to_pandas()
        #pdb.set_trace()
        if wcs is not None:
            if (cattype == 'mean') | (cattype == 'stack'):
                xim,yim=wcs.wcs_world2pix(df['raMean'],df['decMean'],1)
            elif cattype == 'detection':
                xim,yim=wcs.wcs_world2pix(df['ra'],df['dec'],1)
            else:
                raise ValueError(cattype+' is not supported!')
            xlim,ylim=[self.size,self.size]
            xim=np.asarray(np.round(xim),dtype=int)
            yim=np.asarray(np.round(yim),dtype=int)
            df.insert(df.columns.size,'xim',xim)
            df.insert(df.columns.size,'yim',yim)
            cond=(df['xim']>0) & (df['xim']<=xlim-1) & (df['yim']>0) & (df['yim']<=ylim-1)
            dfo=df.loc[cond,:]
        else:
            dfo=df.copy()
        if (cattype == 'mean') | (cattype == 'stack'):
            catname=self.name+'_cat_panstar.fits'
        else:
            catname=self.name+'_'+self.band+'detection_panstar.fits'
        Table.from_pandas(dfo).filled(np.nan).write(self.outdir+catname,overwrite=True)
        return dfo,catname
    
    def checkplot(self,hdu,dfcat):
        data=hdu.data
        hdr=hdu.header
        dfc=dfcat
        #pdb.set_trace()
        # replace NaN values with zero for display
        data[np.isnan(data)] = 0.0
        # set contrast to something reasonable
        transform = AsinhStretch() + PercentileInterval(99.5)
        bdata = transform(data)

        showfile=self.outdir+self.name+'_panstar.png'
        figsize=(14,7)
        fig = plt.figure(figsize=figsize)
        fig.subplots_adjust(left=0.1, right=0.9, bottom=0.1, top=0.9,wspace=0.2,hspace=0.1)
        ax1 = fig.add_subplot(1, 2, 1, projection=WCS(hdr))
        axformatter='dd:mm:ss'
        im=ax1.imshow(bdata, origin='lower', cmap='gray_r',norm=PowerNorm(gamma=1))
        ax1.coords['ra'].set_axislabel('RA')
        ax1.coords['dec'].set_axislabel('DEC')
        ax1.coords['ra'].set_major_formatter(axformatter)
        ax1.coords['dec'].set_major_formatter(axformatter)
        ax1.set_title(self.name+' PanSTARRS1 '+self.band+' band')
        if 'nDetections' in dfc.columns:
            condp=dfc['nDetections']>1
            ax1.scatter(dfc.loc[condp,'xim'],dfc.loc[condp,'yim'],marker='o',s=10,edgecolor='red',facecolor='none')
        else:
            ax1.scatter(dfc['xim'],dfc['yim'],marker='o',s=10,edgecolor='red',facecolor='none')
    
        if self.band3color is not None:
            cim = dpi(ra=self.ra,dec=self.dec,size=self.size,filters=self.band3color,
                      imgtype=self.imgtype).getcolorim()
            ax2 = fig.add_subplot(1, 2, 2, projection=WCS(hdr))
            im=ax2.imshow(cim, origin='lower')
            ax2.coords['ra'].set_axislabel('RA')
            ax2.coords['dec'].set_axislabel(' ')
            ax2.coords['ra'].set_major_formatter(axformatter)
            ax2.coords['dec'].set_major_formatter(axformatter)
            ax2.set_title(self.name+' grz RGB image')


        plt.savefig(showfile,bbox_inches = "tight")
        plt.close()
        #pylab.figure(1,(12,6))
        #pylab.subplot(121)
        #pylab.imshow(gim,origin="upper")
        #pylab.title('Crab Nebula PS1 r (jpeg)')

        #pylab.subplot(122)
        #pylab.title('Crab Nebula PS1 r (fits)')
        #pylab.imshow(bfim,cmap="gray",origin="lower")

        #pylab.savefig(imgdir+'test_fits.png')
if __name__ == '__main__':
    imgdir='/Users/miaomiaozhang/Science/data/WFST-pipeline/test/detection/image/'
    l=60.
    #b=1.5
    #b=15.
    b=0.5
    coo_gal=SkyCoord(l*u.degree,b*u.degree,frame='galactic')
    coo_icrs=coo_gal.transform_to('icrs')
    size = 1280
    #name='Crab_Nebula'
    #name='test_medium'
    name='test_high'
    imgtype='stack'
    outdir='./test/'
    
    dp=download_panstar(coo_icrs, size, name=name, imgtype=imgtype, band3color='grz', band='r', outdir=imgdir)
    
